import assert from './assert.js'

const CollabMap = class {
	constructor () {
		this.changeHandlers = []
		this.localOps = new WeakSet()
		this.ops = new Map()
	}
	delete (key) {
		this.set(key, '')
	}
	get (key) {
		const opOrUndefined = this.ops.get(key)
		return opOrUndefined === undefined ? '' : opOrUndefined.value
	}
	getVersion (key) {
		const opOrUndefined = this.ops.get(key)
		return opOrUndefined === undefined ? 0 : opOrUndefined.version
	}
	has (key) {
		const opOrUndefined = this.ops.get(key)
		return opOrUndefined !== undefined && opOrUndefined.value !== ''
	}
	* keys () {
		for (const op of this.ops.values()) {
			if (op.value !== '') {
				yield op.key
			}
		}
	}
	receive (op) {
		const existingOpOrUndefined = this.ops.get(op.key)
		if (
			existingOpOrUndefined === undefined
			|| existingOpOrUndefined.version < op.version
			|| (existingOpOrUndefined.version === op.version && existingOpOrUndefined.value < op.value)
		) {
			this.ops.set(op.key, op)
			const change = {
				key: op.key,
				oldValue: existingOpOrUndefined === undefined ? '' : existingOpOrUndefined.value,
				op: op,
				value: op.value,
			}
			for (const handler of this.changeHandlers) {
				handler(change)
			}
		}
	}
	set (key, value) {
		assert(typeof key === 'string')
		assert(typeof value === 'string')
		const existingOpOrUndefined = this.ops.get(key)
		const op = {
			key: key,
			value: value,
			version: existingOpOrUndefined === undefined ? 1 : existingOpOrUndefined.version + 1,
		}
		this.localOps.add(op)
		this.ops.set(key, op)
		const change = {
			key: key,
			oldValue: existingOpOrUndefined === undefined ? '' : existingOpOrUndefined.value,
			op: op,
			value: value,
		}
		for (const handler of this.changeHandlers) {
			handler(change)
		}
	}
	* [Symbol.iterator] () {
		for (const op of this.ops.values()) {
			if (op.value !== '') {
				yield [op.key, op.value]
			}
		}
	}
}

export default CollabMap
