import qcEnableLikeButton from './qcEnableLikeButton.js'
import QcRecentsPositions from './QcRecentsPositions.js'
import QcMediaIsLiked from './QcMediaIsLiked.js'
import AnimatedWeakMap from './AnimatedWeakMap.js'
import qcUnbumpPlaylist from './qcUnbumpPlaylist.js'
import qcQueueMap from './qcQueueMap.js'
import qcBumpPlaylist from './qcBumpPlaylist.js'
import CollabPosition from './CollabPosition.js'
import AnimatedRefFn from './AnimatedRefFn.js'
import QcQueueIsPlaying from './QcQueueIsPlaying.js'
import qcBumpTrack from './qcBumpTrack.js'
import qcUnbumpTrack from './qcUnbumpTrack.js'
import QcMainLiContextButton from './QcMainLiContextButton.js'
import QcMainLi from './QcMainLi.js'
import QcQueuePositions from './QcQueuePositions.js'
import QcsMediaExists from './QcsMediaExists.js'
import qcsCollabMap from './qcsCollabMap.js'
import CollabId from './CollabId.js'
import QcTitleBarButton from './QcTitleBarButton.js'
import QcTitleBarForm from './QcTitleBarForm.js'
import QcTitleBar from './QcTitleBar.js'
import QcMainUl from './QcMainUl.js'
import QcMain from './QcMain.js'
import QcTitleBarInput from './QcTitleBarInput.js'
import QcButton from './QcButton.js'
import animators from './animators.js'
import setElChild from './setElChild.js'
import AnimatedRef from './AnimatedRef.js'
import qcFetchOkJsonOrNull from './qcFetchOkJsonOrNull.js'
import QcShouldShowYtResult from './QcShouldShowYtResult.js'
import QueryString from './QueryString.js'

const videoResultTracks = new AnimatedWeakMap()
const ExistingTrack = (videoResult) => {
	if (videoResultTracks.has(videoResult)) {
		const track = videoResultTracks.get(videoResult)
		if (QcsMediaExists(track)) {
			return track
		}
	}
	const track = CollabId()
	qcsCollabMap.set('media-' + track + '-title', videoResult.title)
	if (videoResult.subtitleOrNull !== null) {
		qcsCollabMap.set('media-' + track + '-subtitle', videoResult.subtitleOrNull)
	}
	qcsCollabMap.set('media-' + track + '-duration', JSON.stringify(videoResult.value.duration))
	qcsCollabMap.set('media-' + track + '-info2', JSON.stringify({ytVideo: videoResult.value.video}))
	videoResultTracks.set(videoResult, track)
	return track
}
const VideoLikeButton = (result) => {
	const button = QcButton()
	button.addEventListener('click', () => {
		const track = ExistingTrack(result)
		if (QcMediaIsLiked(track)) {
			qcsCollabMap.set('media-' + track + '-isLiked', JSON.stringify(false))
			qcsCollabMap.delete('media-' + track + '-recentsInfo')
		}
		else {
			qcsCollabMap.set('media-' + track + '-isLiked', JSON.stringify(true))
			const recentsPositions = QcRecentsPositions()
			qcsCollabMap.set('media-' + track + '-recentsInfo', JSON.stringify({position: CollabPosition(null, recentsPositions.length === 0 ? null : recentsPositions[0]), time: Date.now()}))
		}
		// this recents logic won't work once you start deduping...and it's inconsistent with the other like button behaviors which might make it confusing...so should be rethought at some point
	})
	animators.set(button, () => {
		button.textContent = videoResultTracks.has(result) && QcsMediaExists(videoResultTracks.get(result)) && QcMediaIsLiked(videoResultTracks.get(result)) ? '🖤' : '🤍'
	})
	return button
}
const VideoBumpButton = (result) => {
	const button = QcButton()
	button.addEventListener('click', () => {
		qcBumpTrack(ExistingTrack(result))
	})
	button.appendChild(document.createTextNode('🔼'))
	return button
}
const VideoUnbumpButton = (result) => {
	const button = QcButton()
	button.addEventListener('click', () => {
		if (videoResultTracks.has(result)) {
			qcUnbumpTrack(videoResultTracks.get(result))
		}
	})
	animators.set(button, () => {
		button.style.display = videoResultTracks.has(result) && QcsMediaExists(videoResultTracks.get(result)) && qcsCollabMap.has('media-' + videoResultTracks.get(result) + '-queueInfo') ? '' : 'none'
	})
	button.appendChild(document.createTextNode('🔽'))
	return button
}
const VideoContextButton = (result) => {
	const button = QcMainLiContextButton()
	button.appendChild((() => {
		const queueIndexSpan = document.createElement('span')
		queueIndexSpan.classList.add('mainLiQueueIndexSpan')
		animators.set(queueIndexSpan, () => {
			const indexOrNegativeOne = videoResultTracks.has(result) && qcsCollabMap.has('media-' + videoResultTracks.get(result) + '-queueInfo') ? QcQueuePositions().indexOf(JSON.parse(qcsCollabMap.get('media-' + videoResultTracks.get(result) + '-queueInfo')).position) : -1
			queueIndexSpan.textContent = indexOrNegativeOne === -1 ? '' : '[' + (QcQueueIsPlaying() ? (indexOrNegativeOne === 0 ? 'Playing' : indexOrNegativeOne.toString(10)) : (indexOrNegativeOne + 1).toString(10)) + '] '
		})
		return queueIndexSpan
	})())
	button.appendChild(document.createTextNode(result.title))
	if (result.subtitleOrNull !== null) {
		button.appendChild(document.createTextNode(' '))
		button.appendChild((() => {
			const subtitleSpan = document.createElement('span')
			subtitleSpan.classList.add('mainLiSubtitleSpan')
			subtitleSpan.appendChild(document.createTextNode('(' + result.subtitleOrNull + ')'))
			return subtitleSpan
		})())
	}
	return button
}
const PlaylistLikeButton = (result, PlaylistOrNull, videoResultsOrNullRef) => {
	const button = QcButton()
	const LoadOrNull = AnimatedRefFn(null)
	button.addEventListener('click', async () => {
		if (LoadOrNull() === null) {
			const like = () => {
				// todo factor out existingplaylist logic
				if (PlaylistOrNull() === null || !QcsMediaExists(PlaylistOrNull())) {
					const playlist = CollabId()
					qcsCollabMap.set('media-' + playlist + '-title', result.title)
					if (result.subtitleOrNull !== null) {
						qcsCollabMap.set('media-' + playlist + '-subtitle', result.subtitleOrNull)
					}
					let previousPositionOrNull = null
					for (const videoResult of videoResultsOrNullRef.value) {
						const track = ExistingTrack(videoResult)
						const position = CollabPosition(previousPositionOrNull, null)
						qcsCollabMap.set('media-' + track + '-playlists-' + playlist + '-info', JSON.stringify({position: position}))
						previousPositionOrNull = position
					}
					qcsCollabMap.set('media-' + playlist + '-info2', JSON.stringify({}))
					PlaylistOrNull.set(playlist)
				}
				const playlist = PlaylistOrNull()
				if (QcMediaIsLiked(playlist)) {
					qcsCollabMap.set('media-' + playlist + '-isLiked', JSON.stringify(false))
					qcsCollabMap.delete('media-' + playlist + '-recentsInfo')
				}
				else {
					qcsCollabMap.set('media-' + playlist + '-isLiked', JSON.stringify(true))
					const recentsPositions = QcRecentsPositions()
					qcsCollabMap.set('media-' + playlist + '-recentsInfo', JSON.stringify({position: CollabPosition(null, recentsPositions.length === 0 ? null : recentsPositions[0]), time: Date.now()}))
				}
				// this recents logic won't work once you start deduping...and it's inconsistent with the other like button behaviors which might make it confusing...so should be rethought at some point
			}
			if (videoResultsOrNullRef.value === null) {
				const load = {}
				LoadOrNull.set(load)
				const videoResultsOrNull = await qcFetchOkJsonOrNull('/ytPlaylistVideoResultsOrNull?' + QueryString({playlist: result.value}))
				if (load === LoadOrNull()) {
					LoadOrNull.set(null)
					if (videoResultsOrNull === null) {
						alert('error message')
					}
					else {
						if (videoResultsOrNullRef.value === null) {
							videoResultsOrNullRef.value = videoResultsOrNull
						}
						like()
					}
				}
			}
			else {
				like()
			}
		}
		else {
			LoadOrNull.set(null)
		}
	})
	animators.set(button, () => {
		button.textContent = LoadOrNull() === null ? (PlaylistOrNull() !== null && QcsMediaExists(PlaylistOrNull()) && QcMediaIsLiked(PlaylistOrNull()) ? '🖤' : '🤍') : '⌛'
	})
	return button
}
const PlaylistBumpButton = (result, PlaylistOrNull, videoResultsOrNullRef) => {
	const button = QcButton()
	const LoadOrNull = AnimatedRefFn(null)
	button.addEventListener('click', async () => {
		if (LoadOrNull() === null) {
			const bump = () => {
				if (PlaylistOrNull() === null || !QcsMediaExists(PlaylistOrNull())) {
					const playlist = CollabId()
					qcsCollabMap.set('media-' + playlist + '-title', result.title)
					if (result.subtitleOrNull !== null) {
						qcsCollabMap.set('media-' + playlist + '-subtitle', result.subtitleOrNull)
					}
					let previousPositionOrNull = null
					for (const videoResult of videoResultsOrNullRef.value) {
						const track = ExistingTrack(videoResult)
						const position = CollabPosition(previousPositionOrNull, null)
						qcsCollabMap.set('media-' + track + '-playlists-' + playlist + '-info', JSON.stringify({position: position}))
						previousPositionOrNull = position
					}
					qcsCollabMap.set('media-' + playlist + '-info2', JSON.stringify({}))
					PlaylistOrNull.set(playlist)
				}
				qcBumpPlaylist(PlaylistOrNull())
			}
			if (videoResultsOrNullRef.value === null) {
				const load = {}
				LoadOrNull.set(load)
				const videoResultsOrNull = await qcFetchOkJsonOrNull('/ytPlaylistVideoResultsOrNull?' + QueryString({playlist: result.value}))
				if (load === LoadOrNull()) {
					LoadOrNull.set(null)
					if (videoResultsOrNull === null) {
						alert('error message')
					}
					else {
						if (videoResultsOrNullRef.value === null) {
							videoResultsOrNullRef.value = videoResultsOrNull
						}
						bump()
					}
				}
			}
			else {
				bump()
			}
		}
		else {
			LoadOrNull.set(null)
		}
	})
	animators.set(button, () => {
		button.textContent = LoadOrNull() === null ? '🔼' : '⌛'
	})
	return button
}
const PlaylistUnbumpButton = (result, PlaylistOrNull) => {
	const button = QcButton()
	button.textContent = '🔽'
	button.addEventListener('click', async () => {
		if (PlaylistOrNull() !== null) {
			qcUnbumpPlaylist(PlaylistOrNull())
		}
	})
	animators.set(button, () => {
		const playlistOrNull = PlaylistOrNull()
		button.style.display = playlistOrNull === null ? 'none' : (QcQueuePositions().some((position) => {
			return qcQueueMap.get(position).playlist === playlistOrNull
		}) ? '' : 'none')
	})
	return button
}
const PlaylistContextButton = (result, PlaylistOrNull) => {
	const button = QcMainLiContextButton()
	button.appendChild((() => {
		const queueIndexSpan = document.createElement('span')
		queueIndexSpan.classList.add('mainLiQueueIndexSpan')
		animators.set(queueIndexSpan, () => {
			let text = ''
			if (PlaylistOrNull() !== null) {
				const playlist = PlaylistOrNull()
				const queuePositions = QcQueuePositions()
				const firstIndexOrNegativeOne = queuePositions.findIndex((position) => {
					return qcQueueMap.get(position).playlist === playlist
				})
				if (firstIndexOrNegativeOne !== -1) {
					const firstIndex = firstIndexOrNegativeOne
					if (firstIndex === 0 && QcQueueIsPlaying()) {
						text = '[Playing] '
					}
					else {
						let lastIndex
						for (let i = queuePositions.length - 1; i >= 0; i--) {
							if (qcQueueMap.get(queuePositions[i]).playlist === playlist) {
								lastIndex = i
								break
							}
						}
						const AdjustedIndex = (index) => {
							return QcQueueIsPlaying() ? index : index + 1
						}
						text = '[' + AdjustedIndex(firstIndex) + (firstIndex === lastIndex ? '' : '-' + AdjustedIndex(lastIndex)) + '] '
					}
				}
			}
			queueIndexSpan.textContent = text
		})
		return queueIndexSpan
	})())
	button.appendChild(document.createTextNode(result.title))
	if (result.subtitleOrNull !== null) {
		button.appendChild(document.createTextNode(' '))
		button.appendChild((() => {
			const subtitleSpan = document.createElement('span')
			subtitleSpan.classList.add('mainLiSubtitleSpan')
			subtitleSpan.appendChild(document.createTextNode('(' + result.subtitleOrNull + ')'))
			return subtitleSpan
		})())
	}
	return button
}
const ChannelContextButton = (result) => {
	const button = QcMainLiContextButton()
	button.appendChild(document.createTextNode(result.title))
	if (result.subtitleOrNull !== null) {
		button.appendChild(document.createTextNode(' '))
		button.appendChild((() => {
			const subtitleSpan = document.createElement('span')
			subtitleSpan.classList.add('mainLiSubtitleSpan')
			subtitleSpan.appendChild(document.createTextNode('(' + result.subtitleOrNull + ')'))
			return subtitleSpan
		})())
	}
	return button
}
const QcYtResultLi = (tabAfterTick, pageAfterTick, result) => {
	const li = QcMainLi()
	const isVideoChannelSectionResult = result.kind === 'channelSection' && result.results.every((childResult) => {
		return childResult.kind === 'video'
	})
	const videoResultsOrNullRef = new AnimatedRef(isVideoChannelSectionResult ? result.results : null)
	const isExpandedRef = new AnimatedRef(false)
	li.appendChild((() => {
		const row = document.createElement('div')
		row.classList.add('mainLiRow')
		if (result.kind === 'channel' || (result.kind === 'channelSection' && !isVideoChannelSectionResult) || result.kind === 'channelTab') {
			row.appendChild(ChannelContextButton(result))
			row.appendChild((() => {
				const forwardButton = QcButton()
				const loadOrNullRef = new AnimatedRef(null)
				let childResultsPageOrNull = null
				queueMicrotask(() => {
					const tab = tabAfterTick
					const page = pageAfterTick
					forwardButton.addEventListener('click', async () => {
						if (loadOrNullRef.value === null) {
							if (childResultsPageOrNull === null) {
								const load = {}
								loadOrNullRef.value = load
								let childResultsOrNull
								if (result.kind === 'channel') {
									childResultsOrNull = await qcFetchOkJsonOrNull('/ytChannelResultsOrNull3?' + QueryString({channel: result.value, channelTitle: result.title}))
								}
								else if (result.kind === 'channelSection') {
									childResultsOrNull = result.results
								}
								else if (result.kind === 'channelTab') {
									childResultsOrNull = await qcFetchOkJsonOrNull('/ytChannelTabResultsOrNull?' + QueryString({channel: result.channel, channelTitle: result.subtitleOrNull, tabName: result.tabName}))
								}
								if (load === loadOrNullRef.value) {
									loadOrNullRef.value = null
									if (childResultsOrNull === null) {
										alert('error message')
									}
									else {
										const childResults = childResultsOrNull
										const childResultsPage = {
											titleBar: (() => {
												const bar = QcTitleBar()
												bar.appendChild((() => {
													const form = QcTitleBarForm()
													form.appendChild((() => {
														const input = QcTitleBarInput()
														input.disabled = true
														input.value = result.title + (result.subtitleOrNull === null ? '' : ' (' + result.subtitleOrNull + ')')
														return input
													})())
													return form
												})())
												bar.appendChild((() => {
													const backButton = QcTitleBarButton()
													backButton.addEventListener('click', () => {
														tab.pageRef.value = page
													}, {passive: true})
													backButton.appendChild(document.createTextNode('⬅️'))
													return backButton
												})())
												return bar
											})(),
											main: null,
										}
										childResultsPage.main = (() => {
											const main = QcMain()
											if (childResults.length === 0) {
												main.appendChild(document.createTextNode((() => {
													if (result.kind === 'channel') {
														return 'empty channel message'
													}
													if (result.kind === 'channelTab') {
														return 'empty channel tab message'
													}
												})()))
											}
											else {
												main.appendChild((() => {
													const ul = QcMainUl()
													for (const childResult of childResults) {
														if (QcShouldShowYtResult(childResult)) {
															ul.appendChild(QcYtResultLi(tab, childResultsPage, childResult))
														}
													}
													return ul
												})())
											}
											return () => {
												return main
											}
										})()
										tab.pageRef.value = childResultsPage
										childResultsPageOrNull = childResultsPage
									}
								}
							}
							else {
								tab.pageRef.value = childResultsPageOrNull
							}
						}
						else {
							loadOrNullRef.value = null
						}
					}, {passive: true})
				})
				const forwardIcon = document.createTextNode('➡️')
				const loadingIcon = document.createTextNode('⌛')
				animators.set(forwardButton, () => {
					setElChild(forwardButton, loadOrNullRef.value === null ? forwardIcon : loadingIcon)
				})
				return forwardButton
			})())
		}
		else if (result.kind === 'playlist' || isVideoChannelSectionResult) {
			row.appendChild((() => {
				const expandButton = QcButton()
				const loadOrNullRef = new AnimatedRef(null)
				expandButton.addEventListener('click', async () => {
					if (loadOrNullRef.value === null) {
						if (videoResultsOrNullRef.value === null) {
							const load = {}
							loadOrNullRef.value = load
							const videoResultsOrNull = await qcFetchOkJsonOrNull('/ytPlaylistVideoResultsOrNull?' + QueryString({playlist: result.value}))
							if (load === loadOrNullRef.value) {
								loadOrNullRef.value = null
								if (videoResultsOrNull === null) {
									alert('error message')
								}
								else {
									if (videoResultsOrNullRef.value === null) {
										videoResultsOrNullRef.value = videoResultsOrNull
									}
									isExpandedRef.value = true
								}
							}
						}
						else {
							isExpandedRef.value = !isExpandedRef.value
						}
					}
					else {
						loadOrNullRef.value = null
					}
				}, {passive: true})
				const expandIcon = document.createTextNode('➕')
				const loadingIcon = document.createTextNode('⌛')
				const collapseIcon = document.createTextNode('➖')
				animators.set(expandButton, () => {
					setElChild(expandButton, loadOrNullRef.value === null ? (isExpandedRef.value ? collapseIcon : expandIcon) : loadingIcon)
				})
				return expandButton
			})())
			const PlaylistOrNull = AnimatedRefFn(null)
			if (qcEnableLikeButton) {
				row.appendChild(PlaylistLikeButton(result, PlaylistOrNull, videoResultsOrNullRef))
			}
			row.appendChild(PlaylistContextButton(result, PlaylistOrNull))
			row.appendChild(PlaylistUnbumpButton(result, PlaylistOrNull))
			row.appendChild(PlaylistBumpButton(result, PlaylistOrNull, videoResultsOrNullRef))
		}
		else if (result.kind === 'video') {
			if (qcEnableLikeButton) {
				row.appendChild(VideoLikeButton(result))
			}
			row.appendChild(VideoContextButton(result))
			row.appendChild(VideoUnbumpButton(result))
			row.appendChild(VideoBumpButton(result))
		}
		return row
	})())
	if (result.kind === 'playlist' || isVideoChannelSectionResult) {
		li.appendChild((() => {
			const videoResultsUl = document.createElement('ul')
			videoResultsUl.classList.add('mainLiUl')
			let animatedVideoResultsOrNullOrUndefined
			animators.set(videoResultsUl, () => {
				videoResultsUl.style.display = isExpandedRef.value ? '' : 'none'
				const videoResultsOrNull = videoResultsOrNullRef.value
				if (animatedVideoResultsOrNullOrUndefined !== videoResultsOrNull) {
					videoResultsUl.replaceChildren()
					if (videoResultsOrNull !== null) {
						for (const videoResult of videoResultsOrNull) {
							if (QcShouldShowYtResult(videoResult)) {
								videoResultsUl.appendChild(QcYtResultLi(tabAfterTick, pageAfterTick, videoResult))
							}
						}
					}
					animatedVideoResultsOrNullOrUndefined = videoResultsOrNull
				}
			})
			return videoResultsUl
		})())
	}
	return li
}

export default QcYtResultLi
