import YtChannelTabContentResults from './YtChannelTabContentResults.js'
import bufferHttpResBody from './bufferHttpResBody.js'
import HttpResAsJsonValue from './HttpResAsJsonValue.js'
import HttpsGetRes from './HttpsGetRes.js'
import qsWriteDebuggingInfo from './qsWriteDebuggingInfo.js'
import RelativeUriAsUrl from './RelativeUriAsUrl.js'
import ThroweeAsJsonValue from './ThroweeAsJsonValue.js'
import Ungzipped from './Ungzipped.js'
import YtInitialData from './YtInitialData.js'

const TabNameOrNull = (tab) => {
	return tab.tabRenderer === undefined ? null : RelativeUriAsUrl((
		tab
		.tabRenderer
		.endpoint
		.commandMetadata
		.webCommandMetadata
		.url
	)).pathname.split('/').at(-1)
}
const QsYtChannelResultsOrNull = async (channel, channelTitle) => {
	let resOrNull = null
	let resBodyOrNull = null
	const results = []
	let threw = false
	let throweeIfThrew
	try {
		resOrNull = await HttpsGetRes(
			'https://www.youtube.com/channel/' + encodeURIComponent(channel) + '/playlists',
			{headers: {'Accept-Encoding': 'gzip', 'Accept-Language': 'en-US'}},
		)
		resBodyOrNull = await bufferHttpResBody(resOrNull)
		const tabs = (
			YtInitialData((await Ungzipped(resBodyOrNull)).toString())
			.contents
			.twoColumnBrowseResultsRenderer
			.tabs
		)
		if (tabs.some((tab) => {
			return TabNameOrNull(tab) === 'videos'
		})) {
			results.push({
				kind: 'channelTab',
				subtitleOrNull: channelTitle,
				title: 'Recent videos',
				channel: channel,
				tabName: 'videos',
			})
		}
		const playlistsTabOrUndefined = tabs.find((tab) => {
			return TabNameOrNull(tab) === 'playlists'
		})
		if (playlistsTabOrUndefined !== undefined) {
			results.push(...YtChannelTabContentResults(playlistsTabOrUndefined.tabRenderer.content, channelTitle))
		}
		if (tabs.some((tab) => {
			return TabNameOrNull(tab) === 'releases'
		})) {
			results.push({
				kind: 'channelTab',
				subtitleOrNull: channelTitle,
				title: 'Releases',
				channel: channel,
				tabName: 'releases',
			})
		}
		if (results.length === 0) {
			const featuredTabOrUndefined = tabs.find((tab) => {
				return TabNameOrNull(tab) === 'featured'
			})
			if (featuredTabOrUndefined !== undefined) {
				const featuredTab = featuredTabOrUndefined
				if (featuredTab.tabRenderer.content !== undefined) {
					results.push(...YtChannelTabContentResults(featuredTab.tabRenderer.content, channelTitle))
				}
			}
		}
	}
	catch (throwee) {
		threw = true
		throweeIfThrew = throwee
	}
	qsWriteDebuggingInfo('qsYtChannelResultsOrNullInfo', {
		channel: channel,
		channelTitle: channelTitle,
		resBodyOrNull: resBodyOrNull === null ? null : resBodyOrNull.toString('base64'),
		resOrNull: resOrNull === null ? null : HttpResAsJsonValue(resOrNull),
		threw: threw,
		throweeIfThrew: threw ? ThroweeAsJsonValue(throweeIfThrew) : null,
	})
	return threw ? null : results
}

export default QsYtChannelResultsOrNull
