import assert from './assert.js'
import YtTextAsStringId from './YtTextAsStringId.js'
import StringId from './StringId.js'
import YtResultDuration from './YtResultDuration.js'

const YtChannelTabContentResults = (tabContent, channelTitle) => {
	const results = []
	if (tabContent.richGridRenderer !== undefined) {
		for (const content of tabContent.richGridRenderer.contents) {
			if (content.richItemRenderer !== undefined) {
				const itemContent = content.richItemRenderer.content
				if (itemContent.videoRenderer !== undefined) {
					const videoRenderer = itemContent.videoRenderer
					results.push({
						kind: 'video',
						subtitleOrNull: videoRenderer.longBylineText === undefined ? channelTitle : YtTextAsStringId(videoRenderer.longBylineText),
						title: YtTextAsStringId(videoRenderer.title),
						value: {duration: YtResultDuration(videoRenderer.lengthText), video: StringId(videoRenderer.videoId)},
					})
				}
				else if (itemContent.playlistRenderer !== undefined) {
					const playlistRenderer = itemContent.playlistRenderer
					results.push({
						kind: 'playlist',
						subtitleOrNull: YtTextAsStringId(playlistRenderer.longBylineText),
						title: YtTextAsStringId(playlistRenderer.title),
						value: StringId(playlistRenderer.playlistId),
					})
				}

			}
		}
	}
	else if (tabContent.sectionListRenderer !== undefined) {
		for (const sectionListContent of tabContent.sectionListRenderer.contents) {
			if (sectionListContent.itemSectionRenderer !== undefined) {
				for (const itemSectionContent of sectionListContent.itemSectionRenderer.contents) {
					if (itemSectionContent.gridRenderer !== undefined) {
						const playlistResults = []
						for (const item of itemSectionContent.gridRenderer.items) {
							if (item.gridPlaylistRenderer !== undefined) {
								playlistResults.push({
									kind: 'playlist',
									subtitleOrNull: item.gridPlaylistRenderer.longBylineText === undefined ? channelTitle : YtTextAsStringId(item.gridPlaylistRenderer.longBylineText),
									title: YtTextAsStringId(item.gridPlaylistRenderer.title),
									value: StringId(item.gridPlaylistRenderer.playlistId),
								})
							}
						}
						results.push({
							kind: 'channelSection',
							subtitleOrNull: channelTitle,
							title: StringId((
								tabContent
								.sectionListRenderer
								.subMenu
								.channelSubMenuRenderer
								.contentTypeSubMenuItems
								.find((item) => {
									return item.selected
								})
								.title
							)),
							results: playlistResults,
						})
					}
					else if (itemSectionContent.shelfRenderer !== undefined) {
						const shelfRenderer = itemSectionContent.shelfRenderer
						const content = shelfRenderer.content
						if (content.expandedShelfContentsRenderer !== undefined) {
							results.push({
								kind: 'channelSection',
								subtitleOrNull: channelTitle,
								title: YtTextAsStringId(shelfRenderer.title),
								results: content.expandedShelfContentsRenderer.items.map((item) => {
									if (item.playlistRenderer !== undefined) {
										return {
											kind: 'playlist',
											subtitleOrNull: YtTextAsStringId(item.playlistRenderer.longBylineText),
											title: YtTextAsStringId(item.playlistRenderer.title),
											value: StringId(item.playlistRenderer.playlistId),
										}
									}
									if (item.channelRenderer !== undefined) {
										return {
											kind: 'channel',
											subtitleOrNull: null,
											title: YtTextAsStringId(item.channelRenderer.title),
											value: StringId(item.channelRenderer.channelId),
										}
									}
									assert(false)
								}),
							})
						}
						else if (content.horizontalListRenderer !== undefined) {
							results.push({
								kind: 'channelSection',
								subtitleOrNull: channelTitle,
								title: YtTextAsStringId(shelfRenderer.title),
								results: content.horizontalListRenderer.items.map((item) => {
									if (item.gridPlaylistRenderer !== undefined) {
										return {
											kind: 'playlist',
											subtitleOrNull: item.gridPlaylistRenderer.longBylineText === undefined ? channelTitle : YtTextAsStringId(item.gridPlaylistRenderer.longBylineText),
											title: YtTextAsStringId(item.gridPlaylistRenderer.title),
											value: StringId(item.gridPlaylistRenderer.playlistId),
										}
									}
									if (item.gridVideoRenderer !== undefined) {
										const videoRenderer = item.gridVideoRenderer
										return {
											kind: 'video',
											subtitleOrNull: videoRenderer.shortBylineText === undefined ? channelTitle : YtTextAsStringId(videoRenderer.shortBylineText),
											title: YtTextAsStringId(videoRenderer.title),
											value: {duration: YtResultDuration(videoRenderer.thumbnailOverlays.find((overlay) => {
												return overlay.thumbnailOverlayTimeStatusRenderer !== undefined
											}).thumbnailOverlayTimeStatusRenderer.text), video: StringId(videoRenderer.videoId)},
										}
									}
									if (item.gridChannelRenderer !== undefined) {
										return {
											kind: 'channel',
											subtitleOrNull: null,
											title: YtTextAsStringId(item.gridChannelRenderer.title),
											value: StringId(item.gridChannelRenderer.channelId),
										}
									}
									assert(false)
								}),
							})
						}
					}
				}
			}
		}
	}
	return results
}

export default YtChannelTabContentResults
