import QcBumpNumber from './QcBumpNumber.js'
import QcBumpGroups from './QcBumpGroups.js'
import qcSkipForward from './qcSkipForward.js'
import QcsMediaExists from './QcsMediaExists.js'
import qcsCollabMap from './qcsCollabMap.js'
import QcQueuePositions from './QcQueuePositions.js'
import QcQueueIsPlaying from './QcQueueIsPlaying.js'
import assert from './assert.js'
import CollabPosition from './CollabPosition.js'

const qcBumpTrack = (track) => {
	let skippedForward = false
	if (QcsMediaExists(track)) {
		const queueInfoAsJsonOrEmptyString = qcsCollabMap.get('media-' + track + '-queueInfo')
		const positions = QcQueuePositions()
		if (queueInfoAsJsonOrEmptyString === '') {
			if (positions.length === 0) {
				QcQueueIsPlaying.set(false)
			}
			const newPosition = CollabPosition(positions.length === 0 ? null : positions[positions.length - 1], null)
			qcsCollabMap.set('media-' + track + '-queueInfo', JSON.stringify({
				position: newPosition,
				bumpNumber: QcBumpNumber(newPosition, QcBumpGroups()),
			}))
		}
		else {
			const currentPosition = JSON.parse(queueInfoAsJsonOrEmptyString).position
			const currentIndex = positions.indexOf(currentPosition)
			assert(currentIndex !== -1)
			if (currentIndex === 0) {
				if (!QcQueueIsPlaying()) {
					QcQueueIsPlaying.set(true)
				}
			}
			else if (currentIndex === 1 && QcQueueIsPlaying()) {
				qcSkipForward()
				skippedForward = true
			}
			else {
				const groups = QcBumpGroups()
				// confusingly in this context a position is not "in" a group if position === group.positions[0], instead it's considered to be "in" the previous group, this is so bumping is never a no-op from the user's perspective - it always moves it up in the queue, it doesn't just change its bump number which is hidden state
				const groupIndex = groups.findLastIndex((group) => {
					return group.positions.some((groupPosition) => {
						return currentPosition > groupPosition
					})
				})
				assert(groupIndex !== -1) // if groupIndex === -1, currentIndex === 0 and that case has already been handled
				const newPosition = groupIndex === 0 ? CollabPosition(null, positions[0]) : CollabPosition(groups[groupIndex - 1].positions.at(-1), groups[groupIndex].positions[0])
				qcsCollabMap.set('media-' + track + '-queueInfo', JSON.stringify({
					position: newPosition,
					bumpNumber: QcBumpNumber(newPosition, QcBumpGroups({withoutPositions: new Set([currentPosition])})),
				}))
			}
		}
	}
	return {skippedForward: skippedForward}
}

export default qcBumpTrack
