import qcSkipForward from './qcSkipForward.js'
import QcQueueVolume from './QcQueueVolume.js'
import QcQueueIsPlaying from './QcQueueIsPlaying.js'
import QcsMediaInfoOrNull from './QcsMediaInfoOrNull.js'
import qcsCollabMap from './qcsCollabMap.js'
import QueryString from './QueryString.js'
import qcQueueMap from './qcQueueMap.js'
import QcQueuePositions from './QcQueuePositions.js'
import DateAsJsonValue from './DateAsJsonValue.js'
import qcSetupArg from './qcSetupArg.js'
import qcWriteDebuggingInfo from './qcWriteDebuggingInfo.js'
import ThroweeAsJsonValue from './ThroweeAsJsonValue.js'

import './qcCollabMapSetup.js'
import './qcDomSetup.js'
import './qcWsSetup.js'

window.addEventListener('error', async (event) => {
	try {
		// todo ErrorEventAsJsonValue, see https://developer.mozilla.org/en-US/docs/Web/API/ErrorEvent
		await qcWriteDebuggingInfo('qcError', ThroweeAsJsonValue(event.error))
	}
	catch (throwee) {
	}
}, {passive: true})
window.addEventListener('unhandledrejection', async (event) => {
	try {
		await qcWriteDebuggingInfo('qcPromiseRejectionReason', ThroweeAsJsonValue(event.reason))
	}
	catch (throwee) {
	}
}, {passive: true})
qcWriteDebuggingInfo('qcInfo', {
	date: DateAsJsonValue(new Date()),
	locationHref: location.href,
	navigatorUserAgent: navigator.userAgent,
	qcSetupArgQsCollabThread: qcSetupArg.qsCollabThread,
	isSecureContext: isSecureContext,
})
if (location.hostname === 'localhost') {
	const audio = new Audio()
	audio.addEventListener('play', () => {
		if (!QcQueueIsPlaying()) {
			QcQueueIsPlaying.set(true)
		}
	})
	audio.addEventListener('pause', () => {
		if (QcQueueIsPlaying() && !audio.ended) { // in some browsers paused is set to true and pause event fires when audio ends ¯\_(ツ)_/¯
			QcQueueIsPlaying.set(false)
		}
	})
	audio.addEventListener('ended', () => {
		qcSkipForward()
	})
	let currentTrackOrNull = null
	let audioUpdateIsQueued = false
	const queueAudioUpdate = () => {
		if (!audioUpdateIsQueued) {
			queueMicrotask(() => {
				audioUpdateIsQueued = false
				const positions = QcQueuePositions()
				const newTrackOrNull = positions.length === 0 ? null : qcQueueMap.get(positions[0]).track
				if (currentTrackOrNull !== newTrackOrNull) {
					audio.src = newTrackOrNull === null ? '' : '/ytTrackAudio2?' + QueryString({track: newTrackOrNull, video: QcsMediaInfoOrNull(newTrackOrNull).ytVideo})
					currentTrackOrNull = newTrackOrNull
				}
				if (QcQueueIsPlaying()) {
					if (audio.paused) {
						audio.play()
					}
				}
				else {
					if (!audio.paused) {
						audio.pause()
					}
				}
				audio.volume = QcQueueVolume()
			})
			audioUpdateIsQueued = true
		}
	}
	queueAudioUpdate()
	qcsCollabMap.changeHandlers.push(queueAudioUpdate)
}
