import QcBumpGroups from './QcBumpGroups.js'
import QcBumpNumber from './QcBumpNumber.js'
import QcQueuePlayInfo from './QcQueuePlayInfo.js'
import assert from './assert.js'
import qcPlayHistoryMap from './qcPlayHistoryMap.js'
import QcPlayHistoryPositions from './QcPlayHistoryPositions.js'
import qcQueueMap from './qcQueueMap.js'
import qcsCollabMap from './qcsCollabMap.js'
import QcQueuePositions from './QcQueuePositions.js'
import QcQueueIsPlaying from './QcQueueIsPlaying.js'
import CollabPosition from './CollabPosition.js'

const skipBackPlaylist = (queuePositions, playlist) => {
	const skipBackValues = QcPlayHistoryPositions().map((position) => {
		return qcPlayHistoryMap.get(position)
	})
	for (let i = skipBackValues.length - 1; i >= 0; i--) {
		const previousPlaylistOrUndefined = skipBackValues[i].playlist
		if (playlist !== previousPlaylistOrUndefined) {
			const previousTrack = skipBackValues[i].track
			const skipBackInfo = JSON.parse(qcsCollabMap.get(previousPlaylistOrUndefined === undefined ? 'media-' + previousTrack + '-skipBackInfo' : 'media-' + previousTrack + '-playlists-' + previousPlaylistOrUndefined + '-skipBackInfo'))
			assert(queuePositions.length !== 0)
			qcsCollabMap.delete(previousPlaylistOrUndefined === undefined ? 'media-' + previousTrack + '-skipBackInfo' : 'media-' + previousTrack + '-playlists-' + previousPlaylistOrUndefined + '-skipBackInfo')
			qcsCollabMap.set(
				previousPlaylistOrUndefined === undefined ? 'media-' + previousTrack + '-queueInfo' : 'media-' + previousTrack + '-playlists-' + previousPlaylistOrUndefined + '-queueInfo',
				JSON.stringify({...skipBackInfo, position: CollabPosition(null, queuePositions[0])}),
			)
			return true
		}
	}
	return false
}
const qcUnbumpPlaylist = (playlist) => {
	const positions = QcQueuePositions()
	const firstIndexOrNegativeOne = positions.findIndex((position) => {
		return qcQueueMap.get(position).playlist === playlist
	})
	if (firstIndexOrNegativeOne !== -1) {
		const firstIndex = firstIndexOrNegativeOne
		let unbumped = false
		if (firstIndex === 0 && QcQueueIsPlaying()) {
			const playInfo = QcQueuePlayInfo()
			if (playInfo.sourcePlaylist === playlist) {
				QcQueueIsPlaying.set(false)
				unbumped = true
			}
			else {
				unbumped = skipBackPlaylist(positions, playlist)
			}
		}
		if (!unbumped) {
			const groups = QcBumpGroups()
			const firstGroupIndex = groups.findIndex((group) => {
				return group.positions.includes(positions[firstIndex])
			})
			assert(firstGroupIndex !== -1)
			const tracksInPlaylist = positions.filter((position) => {
				return qcQueueMap.get(position).playlist === playlist
			}).map((position) => {
				return qcQueueMap.get(position).track
			})
			if (firstGroupIndex === groups.length - 1) {
				for (const track of tracksInPlaylist) {
					qcsCollabMap.delete('media-' + track + '-playlists-' + playlist + '-queueInfo')
				}
			}
			else {
				let afterPosition = groups[firstGroupIndex + 1].positions.at(-1)
				const beforePositionOrNull = firstGroupIndex === groups.length - 2 ? null : groups[firstGroupIndex + 2].positions[0]
				let bumpNumberOrNull = null
				for (const track of tracksInPlaylist) {
					const newPosition = CollabPosition(afterPosition, beforePositionOrNull)
					if (bumpNumberOrNull === null) {
						bumpNumberOrNull = QcBumpNumber(newPosition, QcBumpGroups({withoutPositions: new Set(positions.filter((position) => {
							return qcQueueMap.get(position).playlist === playlist
						}))}))
					}
					qcsCollabMap.set(
						'media-' + track + '-playlists-' + playlist + '-queueInfo',
						JSON.stringify({position: newPosition, bumpNumber: bumpNumberOrNull}),
					)
					afterPosition = newPosition
				}
			}
		}
	}
}

export default qcUnbumpPlaylist
